package com.mycompany.ctfposhbot_a;

import cz.cuni.amis.pogamut.base.agent.navigation.IPathExecutorState;
import cz.cuni.amis.pogamut.base.agent.navigation.IPathFuture;
import cz.cuni.amis.pogamut.base.agent.navigation.PathExecutorState;
import cz.cuni.amis.pogamut.base3d.worldview.object.ILocated;
import cz.cuni.amis.pogamut.base3d.worldview.object.Location;
import cz.cuni.amis.pogamut.sposh.engine.VariableContext;
import cz.cuni.amis.pogamut.sposh.executor.ActionResult;
import cz.cuni.amis.pogamut.sposh.executor.PrimitiveInfo;
import cz.cuni.amis.pogamut.sposh.executor.StateAction;
import cz.cuni.amis.pogamut.ut2004.communication.messages.gbinfomessages.NavPoint;
import cz.cuni.amis.utils.collections.MyCollections;
import cz.cuni.amis.utils.flag.FlagListener;
import java.util.logging.Level;

/**
 * Action that causes bot to run around the map looking for medkits.
 * @author Honza
 */
@PrimitiveInfo(name = "approachFlagbearer", description = "Try to get as close to the flagbearer as possible.")
public class ApproachFlagbearer extends StateAction<CTFContext> {
    
    boolean approaching = false;
    
    public ApproachFlagbearer(CTFContext ctx) {
        super("approachFlagbearer", ctx);
    }
    
    //called when this action is scheduled to run after some other different action
    @Override
    public void init(VariableContext params) {
        //setting non-flooding log level
        ctx.getPathExecutor().getLog().setLevel(Level.WARNING);
    }

    //called each time this action is scheduled to run if previous action equals to this action
    @Override
    public ActionResult run(VariableContext params) {
        // If we were doing something else or if we just finished previous approach attempt
        if (!approaching || !ctx.getPathExecutor().isExecuting()) {
            ctx.getPathExecutor().stop();
            Location flagLocation = ctx.getCtf().getOurFlag().getLocation();
            Location botLocation = ctx.getBot().getLocation();
            if (flagLocation == null) return ActionResult.RUNNING;
            IPathFuture<ILocated> computedPath = ctx.getPathPlanner().computePath(botLocation, flagLocation);
            ctx.getPathExecutor().followPath(computedPath);                    
        }
        return ActionResult.RUNNING;
    }

    // called when some other action was scheduled to run after this action
    @Override
    public void done(VariableContext params) {
        // Reset approaching state, but don't stop!
        // (We want the bot to check the last known position of flagbearer)
        approaching = false;
    }
}
