package com.mycompany.ctfposhbot_a;

import cz.cuni.amis.pogamut.base.agent.navigation.IPathExecutorState;
import cz.cuni.amis.pogamut.base.agent.navigation.PathExecutorState;
import cz.cuni.amis.pogamut.base.communication.worldview.listener.annotation.EventListener;
import cz.cuni.amis.pogamut.base.communication.worldview.listener.annotation.ObjectClassEventListener;
import cz.cuni.amis.pogamut.base3d.worldview.object.Location;
import cz.cuni.amis.pogamut.sposh.context.UT2004Context;
import cz.cuni.amis.pogamut.ut2004.agent.module.sensor.WeaponPrefs;
import cz.cuni.amis.pogamut.ut2004.agent.module.utils.TabooSet;
import cz.cuni.amis.pogamut.ut2004.agent.navigation.UT2004PathAutoFixer;
import cz.cuni.amis.pogamut.ut2004.agent.navigation.stuckdetector.UT2004DistanceStuckDetector;
import cz.cuni.amis.pogamut.ut2004.agent.navigation.stuckdetector.UT2004PositionStuckDetector;
import cz.cuni.amis.pogamut.ut2004.agent.navigation.stuckdetector.UT2004TimeStuckDetector;
import cz.cuni.amis.pogamut.ut2004.bot.impl.UT2004Bot;
import cz.cuni.amis.pogamut.ut2004.communication.messages.GBObjectUpdate;
import cz.cuni.amis.pogamut.ut2004.communication.messages.ItemType;
import cz.cuni.amis.pogamut.ut2004.communication.messages.gbinfomessages.Item;
import cz.cuni.amis.pogamut.ut2004.communication.messages.gbinfomessages.NavPoint;
import cz.cuni.amis.utils.flag.FlagListener;
import cz.cuni.amis.pogamut.ut2004.communication.messages.gbcommands.ChangeTeam;
import cz.cuni.amis.pogamut.ut2004.communication.messages.gbinfomessages.FlagInfo.FlagInfoUpdate;

/**
 * Context is a class that funsions like a shared object between all primitives.
 * You may use default {@link UT2004Context} or create custom one, that is extended
 * to fulfill needs of your bot (extended memory, custom map planner and other stuff).
 *
 * Remmeber to call method {@link UT2004Context#initialize() }, otherwise you will
 * end up with uninitialized modules and very likely with {@link NullPointerException}.
 * @author Honza
 */
public class CTFContext extends UT2004Context<UT2004Bot> {
    
    UT2004PathAutoFixer autoFixer;

    public CTFContext(UT2004Bot bot) {
        super("PreyContext", bot);
        
        // IMPORTANT: Various modules of context must be initialized.
        initialize();
        
        // INITIALIZE CUSTOM MODULES
        pathExecutor.addStuckDetector(new UT2004TimeStuckDetector(bot, 3000, 10000)); // if the bot does not move for 3 seconds, consider that it is stuck / if bot waits for more than 10 seconds, consider that is is stuck
        pathExecutor.addStuckDetector(new UT2004PositionStuckDetector(bot));          // watch over the position history of the bot, if the bot does not move sufficiently enough, consider that it is stuck
        pathExecutor.addStuckDetector(new UT2004DistanceStuckDetector(bot));          // watch over distances to target
        
        autoFixer = new UT2004PathAutoFixer(bot, pathExecutor, fwMap, navBuilder);
        
        // Let's add the listener here, because it is the same for all movement
        getPathExecutor().getState().addStrongListener(new FlagListener<IPathExecutorState>() {
            @Override
            public void flagChanged(IPathExecutorState t) {
                if (t.getState() == PathExecutorState.STUCK || t.getState() == PathExecutorState.TARGET_REACHED || t.getState() == PathExecutorState.PATH_COMPUTATION_FAILED ) {
                    //reset and pick new nav point
                    getPathExecutor().stop();
                }                
            }
        });
        
        // Ballance teams somewhat
        bot.getParams().setTeam(random.nextInt()%2);
        
        // DEFINE WEAPON PREFERENCES
        weaponPrefs = getWeaponPrefs();
        weaponPrefs.addGeneralPref(ItemType.MINIGUN, false);
        weaponPrefs.addGeneralPref(ItemType.MINIGUN, true);
        weaponPrefs.addGeneralPref(ItemType.LINK_GUN, true);
        weaponPrefs.addGeneralPref(ItemType.LIGHTNING_GUN, true);
        weaponPrefs.addGeneralPref(ItemType.SNIPER_RIFLE, true);
        weaponPrefs.addGeneralPref(ItemType.SHOCK_RIFLE, true);
        weaponPrefs.addGeneralPref(ItemType.ROCKET_LAUNCHER, true);
        weaponPrefs.addGeneralPref(ItemType.ASSAULT_RIFLE, true);
        weaponPrefs.addGeneralPref(ItemType.FLAK_CANNON, false);
        weaponPrefs.addGeneralPref(ItemType.FLAK_CANNON, true);
        weaponPrefs.addGeneralPref(ItemType.BIO_RIFLE, true); 
    }
}
