package com.mycompany.ctfposhbot_a;

import java.io.IOException;
import java.util.logging.Level;

import cz.cuni.amis.pogamut.sposh.ut2004.StateSposhLogicController;
import cz.cuni.amis.pogamut.ut2004.bot.IUT2004BotController;
import cz.cuni.amis.pogamut.ut2004.bot.impl.UT2004Bot;
import cz.cuni.amis.pogamut.ut2004.communication.messages.gbcommands.Initialize;
import cz.cuni.amis.pogamut.ut2004.communication.messages.gbinfomessages.BotKilled;
import cz.cuni.amis.pogamut.ut2004.utils.UT2004BotRunner;
import cz.cuni.amis.utils.exception.PogamutException;

/**
 * Create simple bot that is standing still and when its health falls under 90,
 * it runs around the map, looking for medkits.
 *
 * @see IUT2004BotController Controller for various methods called during
 * construction of bot.
 * @author Honza
 */
public class CTFLogic extends StateSposhLogicController<UT2004Bot, CTFContext> {

    private String SPOSH_PLAN_RESOURCE = "sposh/plan/BotPlan.lap";

    @Override
    protected String getPlan() throws IOException {
        return getPlanFromResource(SPOSH_PLAN_RESOURCE);
    }

    /**
     * Create context that can be accessed in every state primitive.
     *
     * @return new context of this bot.
     */
    @Override
    protected CTFContext createContext() {
        return new CTFContext(bot);
    }

    /**
     * Initialize command of the bot, called during initial handshake, init can
     * set things like name of bot, its skin, skill, team ect.
     *
     * @see Initialize
     * @return
     */
    @Override
    public Initialize getInitializeCommand() {
        return new Initialize().setName("Czestmyr's CTF Bot");
    }

    @Override
    public void botKilled(BotKilled event) {
    }

    /**
     * Create an {@link ExternalBot} with custom made logic and try to connect
     * to Unreal Server at localhost:3000
     *
     * @param args the command line arguments
     */
    public static void main(String[] args) throws PogamutException {
        new UT2004BotRunner(CTFLogic.class, "Czestmyr's CTF Bot").setMain(true).setLogLevel(Level.WARNING).startAgents(1);
    }
}
