package com.mycompany.ctfposhbot_a;

import cz.cuni.amis.pogamut.base.agent.navigation.IPathExecutorState;
import cz.cuni.amis.pogamut.base.agent.navigation.IPathFuture;
import cz.cuni.amis.pogamut.base.agent.navigation.PathExecutorState;
import cz.cuni.amis.pogamut.base3d.worldview.object.ILocated;
import cz.cuni.amis.pogamut.base3d.worldview.object.Location;
import cz.cuni.amis.pogamut.sposh.engine.VariableContext;
import cz.cuni.amis.pogamut.sposh.executor.ActionResult;
import cz.cuni.amis.pogamut.sposh.executor.PrimitiveInfo;
import cz.cuni.amis.pogamut.sposh.executor.StateAction;
import cz.cuni.amis.pogamut.unreal.communication.messages.UnrealId;
import cz.cuni.amis.pogamut.ut2004.agent.module.sensomotoric.Weapon;
import cz.cuni.amis.pogamut.ut2004.communication.messages.ItemType;
import cz.cuni.amis.pogamut.ut2004.communication.messages.gbinfomessages.Item;
import cz.cuni.amis.pogamut.ut2004.communication.messages.gbinfomessages.NavPoint;
import cz.cuni.amis.utils.collections.MyCollections;
import cz.cuni.amis.utils.flag.FlagListener;
import java.util.*;
import java.util.logging.Level;

@PrimitiveInfo(name = "getSomeWeapons", description = "Try to arm yourself.")
public class GetSomeWeapons extends StateAction<CTFContext> {
    
    public GetSomeWeapons(CTFContext ctx) {
        super("getSomeWeapons", ctx);
    }
    
    //called when this action is scheduled to run after some other different action
    @Override
    public void init(VariableContext params) {
        //setting non-flooding log level
        ctx.getPathExecutor().getLog().setLevel(Level.WARNING);
    }

    //called each time this action is scheduled to run if previous action equals to this action
    @Override
    public ActionResult run(VariableContext params) {
        // Find the nearest weapon and navigate to it
        Map<UnrealId, Item> spawnedWeapons = ctx.getItems().getSpawnedItems(ItemType.Category.WEAPON);
        Map<UnrealId, Item> spawnedAmmo = ctx.getItems().getSpawnedItems(ItemType.Category.AMMO);
        Collection<NavPoint> possibleNavPoints = new LinkedList<NavPoint>();
        
        // Collect only those weapons that you don't have
        for (UnrealId weapId: spawnedWeapons.keySet()) {
            if (!ctx.getWeaponry().hasWeapon(spawnedWeapons.get(weapId).getType())) {
                possibleNavPoints.add(ctx.getItems().getItem(weapId).getNavPoint());
            }
        }
        
        NavPoint nearest = null;
        NavPoint current = ctx.getInfo().getNearestNavPoint();
        float nearestDist = Float.MAX_VALUE;
        for (Iterator<NavPoint> it = possibleNavPoints.iterator(); it.hasNext();) {
            NavPoint np = it.next();
            if (ctx.getFwMap().getDistance(current, np) < nearestDist) {
                nearestDist = ctx.getFwMap().getDistance(current, np);
                nearest = np;
            }
        }
        
        IPathFuture<NavPoint> computedPath = ctx.getFwMap().computePath(current, nearest);
        ctx.getPathExecutor().followPath(computedPath);                    
        return ActionResult.RUNNING;
    }

    // called when some other action was scheduled to run after this action
    @Override
    public void done(VariableContext params) {}
}
