package com.mycompany.ctfposhbot_a;

import cz.cuni.amis.pogamut.base.agent.navigation.IPathExecutorState;
import cz.cuni.amis.pogamut.base.agent.navigation.IPathFuture;
import cz.cuni.amis.pogamut.base.agent.navigation.PathExecutorState;
import cz.cuni.amis.pogamut.base.communication.worldview.object.WorldObjectId;
import cz.cuni.amis.pogamut.base3d.worldview.object.ILocated;
import cz.cuni.amis.pogamut.base3d.worldview.object.Location;
import cz.cuni.amis.pogamut.sposh.engine.VariableContext;
import cz.cuni.amis.pogamut.sposh.executor.ActionResult;
import cz.cuni.amis.pogamut.sposh.executor.PrimitiveInfo;
import cz.cuni.amis.pogamut.sposh.executor.StateAction;
import cz.cuni.amis.pogamut.ut2004.communication.messages.gbinfomessages.NavPoint;
import cz.cuni.amis.utils.collections.MyCollections;
import cz.cuni.amis.utils.flag.FlagListener;
import java.util.Map;
import java.util.logging.Level;

/**
 * Action that causes bot to run around the map looking for medkits.
 * @author Honza
 */
@PrimitiveInfo(name = "retrieveFlag", description = "Return our flag.")
public class RetrieveFlag extends StateAction<CTFContext> {

    public RetrieveFlag(CTFContext ctx) {
        super("retrieveFlag", ctx);
    }
    
    //called when this action is scheduled to run after some other different action
    @Override
    public void init(VariableContext params) {
        //setting non-flooding log level
        ctx.getPathExecutor().getLog().setLevel(Level.WARNING);
    }

    //called each time this action is scheduled to run if previous action equals to this action
    @Override
    public ActionResult run(VariableContext params) {
        if (!ctx.getPathExecutor().isExecuting()) {
            Location flagLocation = ctx.getCtf().getOurFlag().getLocation();
            boolean flagIsVisible = ctx.getVisibility().isVisible(flagLocation);
            if (flagIsVisible) {
                // If we see the flag, move towards it
                IPathFuture<ILocated> computedPath = ctx.getPathPlanner().computePath(ctx.getBot().getLocation(), flagLocation);
                ctx.getPathExecutor().followPath(computedPath);
            } else {
                // If we don't see the flag, let's move to a random navpoint.
                Map<WorldObjectId, NavPoint> allNavPoints = ctx.getWorld().getAll(NavPoint.class);
                NavPoint randomNavPoint = MyCollections.getRandom(allNavPoints.values());
                IPathFuture<NavPoint> computedPath = ctx.getFwMap().computePath(ctx.getInfo().getNearestNavPoint(), randomNavPoint);
                ctx.getPathExecutor().followPath(computedPath);
            }                  
        }
        return ActionResult.RUNNING;
    }

    // called when some other action was scheduled to run after this action
    @Override
    public void done(VariableContext params) {}
}

